@@ -3,6 +3,7 @@
 #include "string.h"
 #include <stdlib.h>
 #include <unistd.h>
+#include <unordered_map>
 
 #include <gsl/gsl_rng.h>
 #include <gsl/gsl_randist.h>
@@ -59,9 +60,13 @@ int seir_model (
     bool dist_param_changed = false;
     bool daily_mode_on = false;
 
+    // Create a map for the beta values to avoid repeated calcuations. <unordered_map> is used because it is faster than the regular map.
+    std::unordered_map<double, double> beta_map;
+
     // Import the time window data into a linked list
     TimeWindow *head_node, *current_node = NULL;
-    head_node = importTimeWindowData(params->total_windows,
+    head_node = importTimeWindowData(params->n_pop,
+                                     params->total_windows,
                                      params->input_r0,
                                      params->input_dist_param,
                                      params->input_m,
@@ -71,12 +76,15 @@ int seir_model (
                                      NULL,  // input_death_rate
                                      NULL,  // input_recov_hosp
                                      params->input_window_length);
+
     current_node = head_node;
 
     int n_pop = params->n_pop;
     int n_times;
     n_times = params->t_max / params->tau;  // t_max = 250, tau = 1
 
+    double *beta;
+
     // Events and Updates:
     int n_events = 10;
     int *n_occur = new int[n_events];
@@ -101,6 +109,7 @@ int seir_model (
     prob_ColSum = nrutil_vector(1, n_pop);
     pop_N = nrutil_vector(1, n_pop);
     census_area = nrutil_vector(1, n_pop);
+    beta = nrutil_dvector(1, n_pop);
 
     // compute prob_ColSum and prob_move based on dist_mat and
     // dist_param_low.
@@ -254,10 +263,6 @@ int seir_model (
                     current_node->window_length = n_times - t;
                 }
 
-                // Min/max for R0
-                r0_slope = current_node->getR0Slope();
-                r0_intercept = current_node->getR0Intercept(t - 1);
-
                 if (dist_param != current_node->dist_param)
                 {
                     dist_param_changed = true;
@@ -288,19 +293,41 @@ int seir_model (
             {
                 params->m = current_node->m;
                 params->imm_frac = current_node->imm_frac;
-                r0 = current_node->r0;
-                params->beta = calculateBeta(r0, params);
                 dist_param = current_node->dist_param;
             }
             else
             {
                 params->m = m_slope * t + m_intercept;
                 params->imm_frac = imm_frac_slope * t + imm_frac_intercept;
-                r0 = r0_slope * t + r0_intercept;
-                params->beta = calculateBeta(r0, params);
                 dist_param = dist_param_slope * t + dist_param_intercept;
             }
 
+            // Calculate beta values for each population
+            for (int this_pop = 1; this_pop <= n_pop; this_pop++) {
+                if (daily_mode_on) {
+                    r0 = current_node->r0[this_pop-1];
+
+                    // Avoid calculateBeta if beta was already calculated for this R0
+                    std::unordered_map<double, double>::const_iterator beta_iter = beta_map.find(r0);
+                    if (beta_iter != beta_map.end()) {
+                        beta[this_pop] = beta_iter->second;
+                    }
+                    else {
+                        beta[this_pop] = calculateBeta(r0, params);
+                        beta_map.insert(std::pair<double, double>(r0, beta[this_pop]));
+                    }
+                }
+                else {
+                    // Min/max for R0
+                    r0_slope = current_node->getR0Slope(this_pop-1);
+                    r0_intercept = current_node->getR0Intercept(this_pop-1, t - 1);
+                    r0 = r0_slope * t + r0_intercept;
+
+                    beta[this_pop] = calculateBeta(r0, params);
+                }
+            }
+            params->beta = beta;
+
             // Only deal with prob_move if dist_param changes
             if (dist_param_changed)
             {
@@ -482,6 +509,8 @@ int seir_model (
     // Clean up memory used by time windows
     clearTimeWindows(head_node);
 
+    gsl_rng_free(rand1);
+
     delete[] n_occur;
     delete[] update_vec;
     delete[] update_vec_migrants;
@@ -492,6 +521,7 @@ int seir_model (
     free_vector(prob_ColSum, 1, n_pop);
     free_vector(pop_N, 1, n_pop);
     free_vector(census_area, 1, n_pop);
+    free_dvector(beta, 1, n_pop);
 
     free_imatrix(S_pop, 1, n_pop, 1, 2);
     free_imatrix(E_pop, 1, n_pop, 1, 2);
@@ -521,7 +551,7 @@ void trans_type_beta(double& beta_scaled, int this_pop, double infect_sum,
     case 1:
         // FREQUENCY-DEPENDENT TRANSMISSION
 
-        beta_scaled = fabs(Params->beta / Params->pop_N[this_pop] *
+        beta_scaled = fabs(Params->beta[this_pop] / Params->pop_N[this_pop] *
           (1 + (noise_temp) / pow(infect_sum, 0.5)));
 
         break;
@@ -530,7 +560,7 @@ void trans_type_beta(double& beta_scaled, int this_pop, double infect_sum,
         // DENSITY-DEPENDENT TRANSMISSION
         //// MONOD EQUATION (to determine rel'n btw beta and raw pop_dens)
         pop_dens = (Params->pop_N[this_pop] / Params->census_area[this_pop]);
-        beta_dens = Params->beta * pop_dens / (Params->dd_trans_monod_k + pop_dens);
+        beta_dens = Params->beta[this_pop] * pop_dens / (Params->dd_trans_monod_k + pop_dens);
 
         beta_scaled = fabs(beta_dens / Params->pop_N[this_pop] *
           (1 + (noise_temp) /  pow(infect_sum, 0.5)));
@@ -980,5 +1010,7 @@ double calculateBeta(float r0, SEIRParamStruct *Params)
         printf("\tDEBUG: beta = %.4f for R0 = %.1f\n\n", root, r0);
     }
 
+    gsl_root_fsolver_free(s);
+
     return root;
 }
